
using System;

namespace Stunt.Nutss
{

	/// <summary>
	/// Message generated by a SignalingContext for signaling network specific communications.
	/// </summary>
	public interface ISignalingMessage
	{

		#region Properties

		/// <summary> Retrieves the sender of the message</summary>
		/// <returns> the source of the message </returns>
		NutssUri Source
		{
			get;
		}
		/// <summary> Retrieves the recipient of message</summary>
		/// <returns> the destination of the message </returns>
		NutssUri Destination
		{
			get;
		}
		/// <summary> Whether the message represents a signaling error.</summary>
		/// <returns> true if message is a signaling error </returns>
		bool Error
		{
			get;
		}
		/// <summary> returns the message payload</summary>
		/// <returns> the payload </returns>
		object Payload
		{
			get;
		}

		#endregion

		#region Features

		/// <summary> Whether the message has user data</summary>
		/// <returns> true if message has a payload </returns>
		bool HasPayload
		{
			get;
		}
		/// <summary> Must return the same hashCode for all messages that are part of the same end-to-end signaling session</summary>
		/// <returns> hashcode of message </returns>
		int GetHashCode();
		/// <summary> Must return true for messages that are part of the same end-to-end signaling session </summary>
		/// <param name="o">message to check equality with </param>
		/// <returns> true if both messages are path of the same signaling session </returns>
		bool Equals(object o);

		#endregion

	}
}